@@ -61,6 +61,7 @@ class Element:
 
         if parent:
             self._parent_id = parent.id
+            self._parent.add_child(self)
         else:
             self._parent_id = parent_id
 
@@ -94,7 +95,14 @@ class Element:
         }
 
     def is_valid(self, path=''):
-        """Determine if element has all data needed by procedural generator."""
+        """
+        Determine if element has all data needed by procedural generator.
+
+        Parameters
+        ----------
+        path : str
+            The path to the workflow.  Used to print the location of the error.
+        """
         path = f"{path}/{self}"
 
         if not self.name:
@@ -307,6 +315,15 @@ class Workflow(Element):
             element.remove_successor(child)
 
     def is_valid(self, path=''):
+        """
+        Determine whether the workflow has all data needed by procedural
+        generator.
+
+        Parameters
+        ----------
+        path : str
+            The path to the workflow.  Used to print the location of the error.
+        """
         if not super().is_valid(path):
             return False
 
@@ -332,16 +349,25 @@ class Process(Element):
     """
     Process data to be used as input to procedural generator.
 
-    See base class for parameters.
+    Parameters
+    ----------
+    metadata : list of hyperthought.metadata.MetadataItem
+        Metadata for the process.
+
+    See base class for other parameters.
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, metadata=None, *args, **kwargs):
         super().__init__(*args, **kwargs)
         # Python ids will be used as keys, to avoid duplicates.
         # Metadata names can be used multiple times, but a given MetadataItem
         # class instance can be used only once.
         self._metadata = {}
 
+        if metadata:
+            for metadata_item in metadata:
+                self.add_metadata_item(metadata_item=metadata_item)
+
     @property
     def metadata(self):
         return list(self._metadata.values())
@@ -397,6 +423,40 @@ class Decision(Element):
         self._successors_yes = {}
         self._successors_no = {}
 
+        # Python ids will be used as keys, to avoid duplicates.
+        # Metadata names can be used multiple times, but a given MetadataItem
+        # class instance can be used only once.
+        self._metadata = {}
+
+    @property
+    def metadata(self):
+        return list(self._metadata.values())
+
+    def add_metadata_item(self, metadata_item):
+        if not isinstance(metadata_item, MetadataItem):
+            raise ValueError("metadata_item must be a MetadataItem")
+
+        self._metadata[id(metadata_item)] = metadata_item
+
+    def remove_metadata_item(self, metadata_item):
+        if not isinstance(metadata_item, MetadataItem):
+            raise ValueError("metadata_item must be a MetadataItem")
+
+        item_id = id(metadata_item)
+
+        if item_id not in self._metadata:
+            return
+
+        self._metadata.pop(item_id)
+
+    def remove_metadata(self):
+        self._metadata = {}
+
+    def get_api_formatted_metadata(self):
+        return [
+            item.to_api_format() for item in self._metadata.values()
+        ]
+
     @property
     def decision_question(self):
         return self._decision_question
@@ -406,7 +466,25 @@ class Decision(Element):
         self._validate_nullable_nonempty_string(value)
         self._decision_question = value
 
+    @property
+    def yes_successors(self):
+        return list(self._successors_yes)
+
+    @property
+    def no_successors(self):
+        return list(self._successors_no)
+
     def add_successor(self, element, successor_type):
+        """
+        Add a successor.
+
+        Parameters
+        ----------
+        element : Element
+            The element that will succeed the current element.
+        successor_type : str
+            The type of successor.  Must be 'yes' or 'no'.
+        """
         self._validate_successor_type(successor_type)
         super().add_successor(element)
 
@@ -416,6 +494,7 @@ class Decision(Element):
             self._successors_no[element.id] = element
 
     def remove_successor(self, element):
+        """Remove a successor."""
         super().remove_successor(element)
 
         if element.id in self._successors_yes:
@@ -431,24 +510,43 @@ class Decision(Element):
             )
 
     def is_valid(self, path=''):
+        """
+        Determine whether a decision is valid.
+
+        Parameters
+        ----------
+        path : str
+            Used to print path to invalid elements.
+        """
         if not super().is_valid(path):
             return False
 
         path = f"{path}/{self}"
 
-        # TODO:  Implement decisions in procedural generator.
-        print(f"Invalid node: {path}.  Decisions not yet implemented.")
-        return False
+        yes_successors = set(self._successors_yes.keys())
+        no_successors = set(self._successors_no.keys())
+        all_successors = set(self._successors.keys())
+
+        if all_successors != yes_successors | no_successors:
+            print(
+                f"Invalid node: {path}.  "
+                "Decision successors are incorrectly specified."
+            )
+            return False
 
         if not self.decision_question:
-            print(f"decision missing question: {path}")
+            print(f"Decision missing question: {path}.")
             return False
 
         return True
 
     def to_json(self):
+        """Convert decision element to JSON."""
         output = super().to_json()
         output['decision_question'] = self.decision_question
+        output['yes_successors'] = list(self._successors_yes.keys())
+        output['no_successors'] = list(self._successors_no.keys())
+        output['metadata'] = self.get_api_formatted_metadata()
         return output
 
 
@@ -708,9 +806,23 @@ class WorkflowAPI(GenericAPI):
                 'client_id': child_document['clientId'],
                 'type': child_document['processType'],
                 'name': child_document['name'],
-                'predecessors': child_document['predecessors'],
-                'successors': child_document['successors'],
+                'predecessors': set(child_document['predecessors']),
+                'successors': set(child_document['successors']),
             }
+
+            if child_document['processType'] == 'decision':
+                yes_successors = set()
+                no_successors = set()
+
+                for successor_id in child_document['decisionPath']:
+                    if child_document['decisionPath'][successor_id] == 'yes':
+                        yes_successors.add(successor_id)
+                    elif child_document['decisionPath'][successor_id] == 'no':
+                        no_successors.add(successor_id)
+
+                child['yes_successors'] = yes_successors
+                child['no_successors'] = no_successors
+
             children.append(child)
 
         children = alg.coordinates.add_coordinates(children)
@@ -853,6 +965,13 @@ class WorkflowAPI(GenericAPI):
                     check_validity=False,
                     permissions=permissions,
                 )
+            elif isinstance(child, Decision):
+                self.create_decision(
+                    decision=child,
+                    update_parent=False,
+                    check_validity=False,
+                    permissions=permissions,
+                )
             else:
                 # TODO:  Raise exception.
                 raise Exception(f"unimplemented child type: {child.type}")
@@ -938,7 +1057,99 @@ class WorkflowAPI(GenericAPI):
             document['metadata'] = process.get_api_formatted_metadata()
             self.update_document(document)
 
-    def add_link(self, source_id, target_id):
+    def create_decision(self, decision, update_parent=True,
+                        check_validity=True, permissions=None):
+        """
+        Create a decision document (as opposed to process or decision).
+
+        Parameters
+        ----------
+        decision : Decision
+            Representation of decision data.
+        update_parent : bool
+            Add process to parent workflow and regenerate the parent's XML.
+            This should only be True if a process is added as a child to an
+            existing workflow.
+        check_validity : bool
+            Check the process object for validity.
+            Used to improve performance of recursive calls. Clients can ignore.
+        permissions : dict or None
+            Document permissions.
+            Used in recursive calls.  Clients can ignore.
+        """
+        if check_validity and not decision.is_valid():
+            raise ValueError(f"invalid process: {decision}")
+
+        if not permissions:
+            permissions = self._get_permissions(element=decision)
+
+        if not decision.parent_id:
+            raise ValueError(f"Process without parent_id: {decision.id}")
+
+        username = self._auth.get_username()
+        create_date = datetime.now().isoformat()
+
+        # Specify decision paths.
+        decision_path = {}
+
+        for successor_id in decision.yes_successors:
+            decision_path[successor_id] = 'yes'
+
+        for successor_id in decision.no_successors:
+            decision_path[successor_id] = 'no'
+
+        payload = {
+            'pk': decision.id,
+            'client_id': decision.client_id,
+            'name': decision.name,
+            'creator': username,
+            'created': create_date,
+            'modifier': username,
+            'modified': create_date,
+            'template': True,
+            'process_type': decision.type,
+            'permissions': permissions,
+            'parent_process': decision.parent_id,
+            'predecessors': [
+                predecessor.id for predecessor in decision.predecessors
+            ],
+            'successors': [
+                successor.id for successor in decision.successors
+            ],
+            'decision_path': decision_path,
+        }
+
+        url = f"{self._base_url}/api/workflow/process/"
+        curried_request = partial(
+            requests.post,
+            url=url,
+            json=payload,
+        )
+        r = self.attempt_api_call(curried_request)
+
+        if r.status_code >= ERROR_THRESHOLD:
+            # NOTE:  This method will throw an exception.
+            self._report_api_error(response=r)
+
+        if update_parent:
+            self._update_parent_workflow(
+                new_child_id=decision.id,
+                parent_id=decision.parent_id,
+            )
+
+        # Separate call needed for metadata.
+        # TODO:  Modify POST endpoint to accept metadata.
+        if decision.metadata:
+            # Get document.
+            # (Can't user r.json() from previous call because documents
+            # are formatted inconsistently.  That return value contains the
+            # entire ML document.)
+            # TODO:  Fix the above (in core-router).
+            document = self.get_document(document_id=decision.id)
+            document['metadata'] = decision.get_api_formatted_metadata()
+            self.update_document(document)
+
+    def add_link(self, source_id, target_id, decision_path=None):
         """
         Add a predecessor-successor relationship between workflow elements.
 
@@ -949,7 +1160,50 @@ class WorkflowAPI(GenericAPI):
             elements.
         target_id : str
             The id of the element at the tail of the edge.
+        decision_path : str or None
+            The path to be followed based on the decision, i.e. 'yes' or 'no'.
+            If None, there will be no decision path and the source will
         """
+        if not isinstance(source_id, str):
+            raise ValueError("source_id must be a string")
+
+        if not isinstance(target_id, str):
+            raise ValueError("target_id must be a string")
+
+        if decision_path is not None and decision_path not in ('yes', 'no'):
+            raise ValueError("decision_path must be 'yes' or 'no' if provided")
+
+        # Raise error if a decision path is specified and the source document
+        # is not a decision, or if no decision path is specified and the source
+        # document is a decision.
+        source_document = self.get_document(document_id=source_id)
+
+        if not source_document:
+            raise ValueError(f"No document exists with id {source_id}")
+
+        if 'processType' not in source_document:
+            raise ValueError(
+                f"Document with id {source_id} is not a valid workflow "
+                "document.  It is missing a processType."
+            )
+
+        if (
+            decision_path is not None
+            and
+            source_document['processType'] != 'decision'
+        ):
+            raise ValueError(f"Document with id {source_id} is not a decision")
+
+        if (
+            decision_path is None
+            and
+            source_document['processType'] == 'decision'
+        ):
+            raise ValueError(
+                "A decision_path must be provided for a link from a decision."
+            )
+
+        # Add the predecessor-successor relationship.
         curried_request = partial(
             requests.post,
             url=(
@@ -963,4 +1217,10 @@ class WorkflowAPI(GenericAPI):
             # NOTE:  This method will throw an exception.
             self._report_api_error(response=r)
 
-        return r.json()
+        # Add the decision path.
+        if decision_path:
+            source_document['decisionPath'][target_id] = decision_path
+            self.update_document(updated_document=source_document)
+
+        # Update the parent canvas to display the link.
+        self.redraw_canvas(workflow_id=source_document['parentProcess'])
