@@ -6,6 +6,7 @@ Authorization class.
 
 import base64
 from datetime import datetime
+from datetime import timedelta
 import json
 from threading import Thread
 from time import sleep
@@ -113,9 +114,10 @@ class Authorization:
         }
 
         # Time to sleep prior to requesting an access token refresh.
-        expiration_time = dateutil.parser.parse(auth_payload['expiresAt'])
+        self._expiration_time = dateutil.parser.parse(
+            auth_payload['expiresAt'])
         current_time = datetime.now(tzlocal())
-        time_delta = expiration_time - current_time
+        time_delta = self._expiration_time - current_time
         self._seconds_to_expiration = time_delta.total_seconds()
 
         # Define a refresh thread without starting it.
@@ -215,7 +217,13 @@ class Authorization:
         # Request object, defined here for post-loop error handling.
         r = None
 
-        while self._seconds_to_expiration >= 0:
+        def remaining_time():
+            """Time remaining until token expiration, in seconds."""
+            return (
+                self._expiration_time - datetime.now(tzlocal())
+            ).total_seconds()
+
+        while remaining_time() >= 0:
             if self._delayed_refresh:
                 sleep_time = max(
                     self._seconds_to_expiration - REFRESH_WINDOW, 0)
@@ -246,7 +254,8 @@ class Authorization:
             self._access_token = auth_info['access_token']
             self._refresh_token = auth_info['refresh_token']
             self._seconds_to_expiration = auth_info['expires_in']
-
+            self._expiration_time = datetime.now(tzlocal()) + timedelta(
+                seconds=self._seconds_to_expiration)
             sleep_time = max(self._seconds_to_expiration - REFRESH_WINDOW, 0)
             sleep(sleep_time)
 
