@@ -26,6 +26,7 @@ class MetadataItem:
     VALID_TYPES = {
         'link', 'string', 'boolean', 'decimal', 'integer', 'dateTime'
     }
+    DEFAULT_TYPE_FOR_NONE_VALUES = 'string'
     DATE_CLASSES = {datetime.datetime, datetime.date, datetime.time}
 
     # Reference:  https://numpy.org/doc/stable/user/basics.types.html
@@ -49,42 +50,103 @@ class MetadataItem:
     }
     NUMPY_BOOL_CLASS = 'bool_'
 
+    # Map Python to XML (metadata) types.
+    # This mapping must follow conversion from numpy types.
+    TYPE_MAP = {
+        'str': {
+            'default': 'string',
+            'all': ('link', 'string',)
+        },
+        'bool': {
+            'default': 'boolean',
+            'all': ('boolean',),
+        },
+        'int': {
+            'default': 'integer',
+            'all': ('integer', 'number', 'numeric'),
+        },
+        'float': {
+            'default': 'decimal',
+            'all': ('decimal', 'float', 'double', 'number', 'numeric'),
+        },
+        'date': {
+            'default': 'date',
+            'all': ('date',),
+        },
+        'time': {
+            'default': 'time',
+            'all': ('time',),
+        },
+        'datetime': {
+            'default': 'dateTime',
+            'all': ('dateTime',),
+        },
+    }
+
     def __init__(self, key, value, units=None, annotation=None, type_=None):
         self.key = key
+        self._set_value_and_type(value, type_)
+        self.units = units
+        self.annotation = annotation
+
+    def _set_value_and_type(self, value, type_):
+        # Use default type where no type has been specified and no type can
+        # be inferred.
+        if value is None and type_ is None:
+            self._value = value
+            self._type = self.DEFAULT_TYPE_FOR_NONE_VALUES
+            return
+
+        # Allow any valid type for a None value.
+        # Default as needed if the type is not valid.
+        if value is None and type_ is not None:
+            self._value = value
+            self._type = (
+                type_ if type_ in self.VALID_TYPES
+                else self.DEFAULT_TYPE_FOR_NONE_VALUES
+            )
+            return
+
+        # At this point, value cannot be None.
+        value_type_name = type(value).__name__
 
         # Convert from Numpy types.
         # NOTE:  Use value, not self.value, since the former will be used to
         #        determine the type.
-        if type(value).__name__ in self.NUMPY_INT_CLASS_NAMES:
+        if value_type_name in self.NUMPY_INT_CLASS_NAMES:
             value = int(value)
-        elif type(value).__name__ in self.NUMPY_FLOAT_CLASS_NAMES:
+        elif value_type_name in self.NUMPY_FLOAT_CLASS_NAMES:
             value = float(value)
-        elif type(value).__name__ in self.NUMPY_STRING_CLASSES:
+        elif value_type_name in self.NUMPY_STRING_CLASSES:
             value = str(value)
-        elif type(value).__name__ == self.NUMPY_BOOL_CLASS:
+        elif value_type_name == self.NUMPY_BOOL_CLASS:
             value = bool(value)
 
-        self.value = value
-        self.units = units
-        self.annotation = annotation
-
         if type_ is None:
-            value_type = type(value)
-
-            if value_type == int:
+            if value_type_name == 'int':
                 type_ = 'integer'
-            elif value_type == float:
+            elif value_type_name == 'float':
                 type_ = 'decimal'
-            elif value_type == str:
+            elif value_type_name == 'str':
                 type_ = 'string'
-            elif value_type == bool:
+            elif value_type_name == 'bool':
                 type_ = 'boolean'
-            elif value_type in self.DATE_CLASSES:
+            elif value_type_name in self.DATE_CLASSES:
                 type_ = 'dateTime'
             else:
-                raise ValueError(f"Unknown value type: {value_type}")
+                raise ValueError(f"Unknown value type: {value_type_name}")
 
-        self.type = type_
+            self._value = value
+            self._type = type_
+            return
+
+        # At this point, neither value nor type_ is None.
+        # Make sure the type matches the value_type.
+        if type_ not in self.TYPE_MAP[value_type_name]['all']:
+            type_ = self.TYPE_MAP[value_type_name]['default']
+
+        self._value = value
+        self._type = type_
 
     @property
     def key(self):
@@ -103,10 +165,7 @@ class MetadataItem:
 
     @value.setter
     def value(self, value):
-        if value is None:
-            raise ValueError("value must not be None")
-
-        self._value = value
+        self._set_value_and_type(value, self._type)
 
     @property
     def units(self):
@@ -136,11 +195,7 @@ class MetadataItem:
 
     @type.setter
     def type(self, value):
-        if value not in self.VALID_TYPES:
-            raise ValueError(
-                f"type_ must be one of {', '.join(self.VALID_TYPES)}")
-
-        self._type = value
+        self._set_value_and_type(value=self._value, type_=value)
 
     def to_api_format(self):
         """
