@@ -16,7 +16,7 @@ class _NotAvailable:
     pass
 
 
-class _NotAvailableException:
+class _NotAvailableException(Exception):
     pass
 
 
